@@ -7,13 +7,14 @@ for enumerating and controlling the loads are supported.
 __author__ = "Dima Zavin"
 __copyright__ = "Copyright 2016, Dima Zavin"
 
+from datetime import timedelta
 from enum import Enum
 import logging
 import socket
-import telnetlib
 import threading
 import time
 
+from pylutron._telnetlib import telnetlib
 from typing import Any, Callable, Dict, Type
 
 _LOGGER = logging.getLogger(__name__)
@@ -504,7 +505,7 @@ class Lutron(object):
   def send(self, op, cmd, integration_id, *args):
     """Formats and sends the requested command to the Lutron controller."""
     out_cmd = ",".join(
-        (cmd, str(integration_id)) + tuple((str(x) for x in args)))
+        (cmd, str(integration_id)) + tuple((str(x) for x in args if x is not None)))
     self._conn.send(op + out_cmd)
 
   def load_xml_db(self, cache_path=None):
@@ -621,12 +622,17 @@ class LutronEntity(object):
   def uuid(self):
     return self._uuid
 
+  @property
+  def legacy_uuid(self):
+    """Return a synthesized uuid."""
+    return None
+
   def _dispatch_event(self, event: LutronEvent, params: Dict):
     """Dispatches the specified event to all the subscribers."""
     for handler, context in self._subscribers:
       handler(self, context, event, params)
 
-  def subscribe(self, handler: LutronEventHandler, context):
+  def subscribe(self, handler: LutronEventHandler, context) -> Callable[[], None]:
     """Subscribes to events from this entity.
 
     handler: A callable object that takes the following arguments (in order)
@@ -636,8 +642,10 @@ class LutronEntity(object):
              params: a dict of event-specific parameters
 
     context: User-supplied, opaque object that will be passed to handler.
+    Returns: A callable that can be used to unsubscribe from the event.
     """
     self._subscribers.append((handler, context))
+    return lambda: self._subscribers.remove((handler, context))
 
   def handle_update(self, args):
     """The handle_update callback is invoked when an event is received
@@ -655,6 +663,7 @@ class Output(LutronEntity):
   switched/dimmed load, e.g. light fixture, outlet, etc."""
   _CMD_TYPE = 'OUTPUT'
   _ACTION_ZONE_LEVEL = 1
+  _ACTION_ZONE_FLASH = 5
 
   class Event(LutronEvent):
     """Output events that can be generated.
@@ -691,6 +700,10 @@ class Output(LutronEntity):
     """The integration id"""
     return self._integration_id
 
+  @property
+  def legacy_uuid(self):
+    return '%d-0' % self.id
+
   def handle_update(self, args):
     """Handles an event update for this object, e.g. dimmer level change."""
     _LOGGER.debug("handle_update %d -- %s" % (self._integration_id, args))
@@ -724,15 +737,31 @@ class Output(LutronEntity):
 
   @level.setter
   def level(self, new_level):
+    """Sets the new output level."""
+    self.set_level(new_level)
+
+  @staticmethod
+  def _fade_time(seconds):
+    if seconds is None:
+      return None
+    return str(timedelta(seconds=seconds))
+
+  def set_level(self, new_level, fade_time_seconds=None):
     """Sets the new output level."""
     if self._level == new_level:
       return
     self._lutron.send(Lutron.OP_EXECUTE, Output._CMD_TYPE, self._integration_id,
-        Output._ACTION_ZONE_LEVEL, "%.2f" % new_level)
+        Output._ACTION_ZONE_LEVEL, "%.2f" % new_level, self._fade_time(fade_time_seconds))
     self._level = new_level
 
-## At some later date, we may want to also specify fade and delay times
-#  def set_level(self, new_level, fade_time, delay):
+  def flash(self, fade_time_seconds=None):
+    """Flashes the zone until a new level is set."""
+    self._lutron.send(Lutron.OP_EXECUTE, Output._CMD_TYPE, self._integration_id,
+        Output._ACTION_ZONE_FLASH, self._fade_time(fade_time_seconds))
+    
+
+## At some later date, we may want to also specify delay times
+#  def set_level(self, new_level, fade_time_seconds, delay):
 #    self._lutron.send(Lutron.OP_EXECUTE, Output._CMD_TYPE,
 #        Output._ACTION_ZONE_LEVEL, new_level, fade_time, delay)
 
@@ -798,6 +827,10 @@ class KeypadComponent(LutronEntity):
     is only used for interfacing with the controller."""
     return self._component_num
 
+  @property
+  def legacy_uuid(self):
+    return '%d-%d' % (self._keypad.id, self._component_num)
+
   def handle_update(self, action, params):
     """Handle the specified action on this component."""
     _LOGGER.debug('Keypad: "%s" Handling "%s" Action: %s Params: %s"' % (
@@ -1000,6 +1033,10 @@ class Keypad(LutronEntity):
     """The integration id"""
     return self._integration_id
 
+  @property
+  def legacy_uuid(self):
+    return '%d-0' % self.id
+
   @property
   def name(self):
     """Returns the name of this keypad"""
@@ -1096,6 +1133,10 @@ class MotionSensor(LutronEntity):
     """The integration id"""
     return self._integration_id
 
+  @property
+  def legacy_uuid(self):
+    return str(self.id)
+
   def __str__(self):
     """Returns a pretty-printed string for this object."""
     return 'MotionSensor {} Id: {} Battery: {} Power: {}'.format(
@@ -1186,13 +1227,18 @@ class OccupancyGroup(LutronEntity):
   def _bind_area(self, area):
     self._area = area
     self._integration_id = area.id
-    self._lutron.register_id(OccupancyGroup._CMD_TYPE, self)
+    if self._integration_id != 0:
+      self._lutron.register_id(OccupancyGroup._CMD_TYPE, self)
 
   @property
   def id(self):
-    """The integration id"""
+    """The integration id, which is the area's integration_id"""
     return self._integration_id
 
+  @property
+  def legacy_uuid(self):
+    return '%s-%s' % (self._area.id, self._group_number)
+
   @property
   def group_number(self):
     """The OccupancyGroupNumber"""
